package com.ruoyi.bst.order.service.impl;

import com.ruoyi.bst.order.domain.Order;
import com.ruoyi.bst.order.domain.OrderBO;
import com.ruoyi.bst.order.domain.OrderQuery;
import com.ruoyi.bst.order.domain.OrderVO;
import com.ruoyi.bst.order.domain.enums.OrderStatus;
import com.ruoyi.bst.order.mapper.OrderMapper;
import com.ruoyi.bst.order.service.OrderAssembler;
import com.ruoyi.bst.order.service.OrderService;
import com.ruoyi.bst.order.service.OrderValidator;
import com.ruoyi.bst.orderProd.domain.OrderProd;
import com.ruoyi.bst.orderProd.domain.OrderProdBO;
import com.ruoyi.bst.orderProd.domain.OrderProdVO;
import com.ruoyi.bst.orderProd.service.IOrderProdService;
import com.ruoyi.bst.prodProcess.domain.ProdProcess;
import com.ruoyi.bst.prodProcess.domain.ProdProcessBO;
import com.ruoyi.bst.prodProcess.domain.ProdProcessVO;
import com.ruoyi.bst.prodProcess.service.IProdProcessService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.utils.collection.DiffListVO;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.system.dept.domain.SysDeptQuery;
import com.ruoyi.system.dept.domain.SysDeptVO;
import com.ruoyi.system.dept.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 生产订单Service业务层处理
 *
 * @author ruoyi
 * @date 2024-12-16
 */
@Service
public class OrderServiceImpl implements OrderService
{
    @Autowired
    private OrderMapper orderMapper;

    @Autowired
    private OrderValidator orderValidator;

    @Autowired
    private IOrderProdService orderProdService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IProdProcessService prodProcessService;

    @Autowired
    private OrderAssembler orderAssembler;

    @Autowired
    private ISysDeptService deptService;

    /**
     * 查询生产订单
     *
     * @param id 生产订单主键
     * @return 生产订单
     */
    @Override
    public OrderVO selectOrderById(Long id)
    {
        if (id == null) {
            return null;
        }
        return orderMapper.selectOrderById(id);
    }

    /**
     * 查询生产订单列表
     *
     * @param order 生产订单
     * @return 生产订单
     */
    @Override
    public List<OrderVO> selectOrderList(OrderQuery order)
    {
        return orderMapper.selectOrderList(order);
    }

    /**
     * 新增生产订单
     *
     * @param order 生产订单
     * @return 结果
     */
    @Override
    public int insertOrder(Order order)
    {
        order.setCreateTime(DateUtils.getNowDate());
        return orderMapper.insertOrder(order);
    }

    /**
     * 修改生产订单
     *
     * @param order 生产订单
     * @return 结果
     */
    @Override
    public int updateOrder(Order order)
    {
        return orderMapper.updateOrder(order);
    }

    /**
     * 批量删除生产订单
     *
     * @param ids 需要删除的生产订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderByIds(Long[] ids)
    {
        return orderMapper.deleteOrderByIds(ids);
    }

    /**
     * 删除生产订单信息
     *
     * @param id 生产订单主键
     * @return 结果
     */
    @Override
    public int deleteOrderById(Long id)
    {
        return orderMapper.deleteOrderById(id);
    }

    @Override
    public int addOrder(OrderBO order) {
        // 前校验
        orderValidator.checkBefore(order);

        Integer result = transactionTemplate.execute(status -> {
            int insert = this.insertOrder(order);

            if (insert == 1) {
                // 批量更新产品明细
                this.batchUpdateProdList(order);

                // 批量更新工序明细
                this.batchUpdateProcessList(order);
            }

            // 后校验
            OrderVO vo = this.selectOrderById(order.getId());
            orderValidator.checkAfter(vo);

            return insert;
        });


        return result == null ? 0 : result;
    }

    @Override
    public int editOrder(OrderBO order) {
        // 前校验
        orderValidator.checkBefore(order);

        Integer result = transactionTemplate.execute(status -> {
            int update = this.updateOrder(order);

            if (update == 1) {
                // 批量更新产品明细
                this.batchUpdateProdList(order);

                // 批量更新工序明细
                this.batchUpdateProcessList(order);
            }

            // 后校验
            OrderVO vo = this.selectOrderById(order.getId());
            orderValidator.checkAfter(vo);

            return update;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int logicDel(Long id) {
        if (id == null) {
            return 0;
        }
        OrderVO old = selectOrderById(id);
        ServiceUtil.assertion(old == null, "待删除的订单不存在");
        ServiceUtil.assertion(!OrderStatus.canDel().contains(old.getStatus()), "当前订单状态不允许删除");

        Integer result = transactionTemplate.execute(status -> {
            // 删除订单
            Order data = new Order();
            data.setDeleted(true);
            OrderQuery query = new OrderQuery();
            query.setId(id);
            query.setStatusList(OrderStatus.canDel());
            query.setDeleted(false);
            int del = this.updateByQuery(data, query);

            // 删除明细
            if (del == 1) {
                orderProdService.logicDelByOrderId(id);

                prodProcessService.logicDelByOrderId(id);
            }

            return del;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int finish(Long id) {
        if (id == null) {
            return 0;
        }
        // 校验
        OrderVO old = selectOrderById(id);
        ServiceUtil.assertion(old == null, "待完工的订单不存在");
        ServiceUtil.assertion(!OrderStatus.canFinished().contains(old.getStatus()), "当前订单状态不允许完工");

        List<OrderVO> list = Collections.singletonList(old);
        orderAssembler.assembleVO(list);
        ServiceUtil.assertion(old.getEndProgress().compareTo(new BigDecimal("100")) < 0, "当前订单进度：%s%%，暂不满足完工条件：100%%", old.getEndProgress());

        // 更新订单状态
        Order data = new Order();
        data.setStatus(OrderStatus.FINISHED.getStatus());
        OrderQuery query = new OrderQuery();
        query.setId(id);
        query.setStatusList(OrderStatus.canFinished());
        return updateByQuery(data, query);
    }

    @Override
    public int selectCount(OrderQuery query) {
        return orderMapper.selectCount(query);
    }

    @Override
    public Map<String, Object> parseExcelData(MultipartFile file) {
        ServiceUtil.assertion(file == null, "请选择需要解析的数据");
        ExcelUtil<OrderProdVO> util = new ExcelUtil<>(OrderProdVO.class);
        try {
            Map<String, Object> data = util.importMixedExcel(file.getInputStream(), 8, 9);

            List<OrderProdVO> body = (List<OrderProdVO>) data.get("body");

            // 查询全部部门
            List<SysDeptVO> depts = deptService.selectDeptList(new SysDeptQuery());

            for (int i = 0; i < body.size(); i++) {
                OrderProdVO item = body.get(i);

                String handleWay = item.getHandleWay();
                if (StringUtils.isBlank(handleWay)) {
                    continue;
                }
                // 将处理方式转为部门数据
                // 通过部门名称或者工序简称来匹配部门
                List<String> handleWayList = Arrays.asList(handleWay.split("\\+"));

                List<ProdProcessVO> processList = new ArrayList<>();
                for (int j = 0; j < handleWayList.size(); j++) {
                    String way = handleWayList.get(j).trim();
                    SysDeptVO dept = depts.stream()
                            .filter(deptItem -> {
                                List<String> deptProcess = new ArrayList<>();
                                if (StringUtils.hasText(deptItem.getProcess())) {
                                    deptProcess = Arrays.asList(deptItem.getProcess().split(","));
                                }
                                return Objects.equals(deptItem.getDeptName(), way) || deptProcess.contains(way);
                            })
                            .findFirst().orElse(null);
                    if (dept != null) {
                        ProdProcessVO process = new ProdProcessVO();
                        process.setDeptId(dept.getDeptId());
                        process.setDeptName(dept.getDeptName());
                        process.setNum(item.getNum());
                        if (j == handleWayList.size() - 1) {
                            process.setIsEnd(true);
                        } else {
                            process.setIsEnd(false);
                        }
                        processList.add(process);
                    }
                }

                String newHandleWay = processList.stream().map(ProdProcessVO::getDeptName).collect(Collectors.joining("+"));
                item.setHandleWay(newHandleWay);
                item.setProcessList(processList);
            }

            return data;
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServiceException(e.getMessage());
        }
    }

    private int updateByQuery(Order data, OrderQuery query) {
        return orderMapper.updateByQuery(data, query);
    }

    private int batchUpdateProcessList(OrderBO bo) {
        if (bo == null || CollectionUtils.isEmptyElement(bo.getProdList())) {
            return 0;
        }
        // 关联产品ID
        for (OrderProdBO prod : bo.getProdList()) {
            for (ProdProcessBO process : prod.getProcessList()) {
                process.setOrderProdId(prod.getId());
            }
        }

        List<ProdProcessVO> oldList = prodProcessService.selectByOrderId(bo.getId());

        List<ProdProcessBO> newList = bo.getProdList().stream().map(OrderProdBO::getProcessList).flatMap(List::stream).collect(Collectors.toList());

        // 分离出新增、修改、删除的列表
        DiffListVO<ProdProcessBO, ProdProcessVO> diff = CollectionUtils.convertToDiffList(newList, oldList, ProdProcess::getId);

        int result = 0;
        if (diff.getAddCount() > 0) {
            int batchInsert = prodProcessService.batchInsert(diff.getAddList());
            ServiceUtil.assertion(batchInsert != diff.getAddCount(), "新增工序明细失败");
            result += batchInsert;
        }
        if (diff.getUpdateCount() > 0) {
            int update = prodProcessService.batchUpdate(diff.getUpdateList());
            ServiceUtil.assertion(update != diff.getUpdateCount(), "修改工序明细失败");
            result += update;
        }
        if (diff.getDelCount() > 0) {
            int del = prodProcessService.batchLogicDel(diff.getDelList());
            ServiceUtil.assertion(del != diff.getDelCount(), "删除工序明细失败");
            result += del;
        }

        return result;
    }

    private int batchUpdateProdList(OrderBO bo) {
        if (bo == null || CollectionUtils.isEmptyElement(bo.getProdList())) {
            return 0;
        }
        // 关联订单ID
        for (OrderProd prod : bo.getProdList()) {
            prod.setOrderId(bo.getId());
        }

        List<OrderProdVO> oldList = orderProdService.selectByOrderId(bo.getId());

        // 分离出新增、修改、删除的列表
        DiffListVO<OrderProdBO, OrderProdVO> diff = CollectionUtils.convertToDiffList(bo.getProdList(), oldList, OrderProd::getId);

        int result = 0;
        if (diff.getAddCount() > 0) {
            int batchInsert = orderProdService.batchInsert(diff.getAddList());
            ServiceUtil.assertion(batchInsert != diff.getAddCount(), "新增产品明细失败");
            result += batchInsert;
        }
        if (diff.getUpdateCount() > 0) {
            int update = orderProdService.batchUpdate(diff.getUpdateList());
            ServiceUtil.assertion(update != diff.getUpdateCount(), "修改产品明细失败");
            result += update;
        }
        if (diff.getDelCount() > 0) {
            int del = orderProdService.batchLogicDel(diff.getDelList());
            ServiceUtil.assertion(del != diff.getDelCount(), "删除产品明细失败");
            result += del;
        }

        return result;
    }
}
